@@ -41,43 +41,49 @@
 
 /* Language definitions. */
 
+/* Supported since gcc 5.1.0 and clang 2.9.0. For attributes that appeared
+ * before these versions, in addition we need to do version checks.  */
+#ifndef __has_attribute
+#define __has_attribute(x)	0
+#endif
+
+#ifndef __has_include
+#define __has_include(x)	0
+#endif
+
 #ifdef __GNUC__
 #define LIBCOMPAT_GCC_VERSION (__GNUC__ << 8 | __GNUC_MINOR__)
 #else
 #define LIBCOMPAT_GCC_VERSION 0
 #endif
 
-#if LIBCOMPAT_GCC_VERSION >= 0x0300
-#define LIBCOMPAT_ATTR_PRINTF(n)	__attribute__((format(printf, n, n + 1)))
-#define LIBCOMPAT_ATTR_VPRINTF(n)	__attribute__((format(printf, n, 0)))
+#if LIBCOMPAT_GCC_VERSION >= 0x0300 || __has_attribute(__format__)
+#define LIBCOMPAT_ATTR_FMT(t, f, a)	__attribute__((__format__(t, f, a)))
+#define LIBCOMPAT_ATTR_PRINTF(n)	LIBCOMPAT_ATTR_FMT(__printf__, n, n + 1)
+#define LIBCOMPAT_ATTR_VPRINTF(n)	LIBCOMPAT_ATTR_FMT(__printf__, n, 0)
 #else
+#define LIBCOMPAT_ATTR_FMT(t, f, a)
 #define LIBCOMPAT_ATTR_PRINTF(n)
 #define LIBCOMPAT_ATTR_VPRINTF(n)
 #endif
 
-#if LIBCOMPAT_GCC_VERSION >= 0x0300
-#define LIBCOMPAT_ATTR_NORET		__attribute__((noreturn))
+#if LIBCOMPAT_GCC_VERSION >= 0x0300 || __has_attribute(__noreturn__)
+#define LIBCOMPAT_ATTR_NORET		__attribute__((__noreturn__))
 #else
 #define LIBCOMPAT_ATTR_NORET
 #endif
 
-#if LIBCOMPAT_GCC_VERSION >= 0x0400
-#define LIBCOMPAT_ATTR_SENTINEL		__attribute__((sentinel))
+#if LIBCOMPAT_GCC_VERSION >= 0x0400 || __has_attribute(__sentinel__)
+#define LIBCOMPAT_ATTR_SENTINEL		__attribute__((__sentinel__))
 #else
 #define LIBCOMPAT_ATTR_SENTINEL
 #endif
 
-/* For C++, define a __func__ fallback in case it's not natively supported. */
-#if defined(__cplusplus) && __cplusplus < 201103L
-# if LIBCOMPAT_GCC_VERSION >= 0x0200
-#  define __func__ __PRETTY_FUNCTION__
-# else
-#  define __func__ __FUNCTION__
-# endif
-#endif
-
-#if defined(__cplusplus) && __cplusplus < 201103L
-#define nullptr 0
+#if __has_attribute(__enum_extensibility__)
+#define LIBCOMPAT_ATTR_ENUM_FLAGS \
+	__attribute__((__enum_extensibility__(closed),__flag_enum__))
+#else
+#define LIBCOMPAT_ATTR_ENUM_FLAGS
 #endif
 
 #ifdef __cplusplus
@@ -139,6 +145,10 @@ extern "C" {
 #define unsetenv test_unsetenv
 #endif
 
+#if !HAVE_DECL_SYS_SIGLIST
+extern const char *const sys_siglist[];
+#endif
+
 #if TEST_LIBCOMPAT || !defined(HAVE_C99_SNPRINTF)
 int snprintf(char *str, size_t n, char const *fmt, ...)
 	LIBCOMPAT_ATTR_PRINTF(3);
