@@ -35,6 +35,61 @@
 #include <dpkg/fdio.h>
 #include <dpkg/buffer.h>
 #include <dpkg/file.h>
+#include <dpkg/execname.h>
+
+/**
+ * Get the current working directory.
+ *
+ */
+void
+file_getcwd(struct varbuf *cwd)
+{
+	varbuf_reset(cwd);
+	varbuf_grow(cwd, 64);
+	while (getcwd(cwd->buf, cwd->size) == NULL)
+		varbuf_grow(cwd, cwd->size * 2);
+	varbuf_trunc(cwd, strlen(cwd->buf));
+}
+
+/**
+ * Read the symlink content into a varbuf.
+ *
+ */
+ssize_t
+file_readlink(const char *slink, struct varbuf *content, size_t content_len)
+{
+	ssize_t linksize;
+
+	varbuf_reset(content);
+	varbuf_grow(content, content_len + 1);
+
+	linksize = readlink(slink, content->buf, content->size);
+	if (linksize < 0)
+		return linksize;
+
+	varbuf_trunc(content, linksize);
+
+	return linksize;
+}
+
+/**
+ * Check whether a filename is executable.
+ *
+ * @param filename The filename to check.
+ */
+bool
+file_is_exec(const char *filename)
+{
+	struct stat st;
+
+	if (stat(filename, &st) < 0)
+		return false;
+
+	if (!S_ISREG(st.st_mode))
+		return false;
+
+	return st.st_mode & 0111;
+}
 
 /**
  * Copy file ownership and permissions from one file to another.
@@ -47,17 +102,17 @@ file_copy_perms(const char *src, const char *dst)
 {
 	struct stat stab;
 
-	if (stat(src, &stab) == -1) {
+	if (stat(src, &stab) < 0) {
 		if (errno == ENOENT)
 			return;
 		ohshite(_("unable to stat source file '%.250s'"), src);
 	}
 
-	if (chown(dst, stab.st_uid, stab.st_gid) == -1)
+	if (chown(dst, stab.st_uid, stab.st_gid) < 0)
 		ohshite(_("unable to change ownership of target file '%.250s'"),
 		        dst);
 
-	if (chmod(dst, (stab.st_mode & ~S_IFMT)) == -1)
+	if (chmod(dst, (stab.st_mode & ~S_IFMT)) < 0)
 		ohshite(_("unable to set mode of target file '%.250s'"), dst);
 }
 
@@ -77,10 +132,10 @@ file_slurp_fd(int fd, const char *filename, struct varbuf *vb,
 	if (st.st_size == 0)
 		return 0;
 
-	varbuf_init(vb, st.st_size);
+	varbuf_init(vb, st.st_size + 1);
 	if (fd_read(fd, vb->buf, st.st_size) < 0)
 		return dpkg_put_errno(err, _("cannot read %s"), filename);
-	vb->used = st.st_size;
+	varbuf_trunc(vb, st.st_size);
 
 	return 0;
 }
@@ -127,7 +182,7 @@ file_unlock(int lockfd, const char *lockfile, const char *lockdesc)
 
 	file_lock_setup(&fl, F_UNLCK);
 
-	if (fcntl(lockfd, F_SETLK, &fl) == -1)
+	if (fcntl(lockfd, F_SETLK, &fl) < 0)
 		ohshite(_("unable to unlock %s"), lockdesc);
 }
 
@@ -154,7 +209,7 @@ file_is_locked(int lockfd, const char *filename)
 
 	file_lock_setup(&fl, F_WRLCK);
 
-	if (fcntl(lockfd, F_GETLK, &fl) == -1)
+	if (fcntl(lockfd, F_GETLK, &fl) < 0)
 		ohshit(_("unable to check file '%s' lock status"), filename);
 
 	if (fl.l_type == F_WRLCK && fl.l_pid != getpid())
@@ -188,8 +243,9 @@ file_lock(int *lockfd, enum file_lock_flags flags, const char *filename,
 	else
 		lock_cmd = F_SETLK;
 
-	if (fcntl(*lockfd, lock_cmd, &fl) == -1) {
+	if (fcntl(*lockfd, lock_cmd, &fl) < 0) {
 		const char *warnmsg;
+		char *execname;
 
 		if (errno != EACCES && errno != EAGAIN)
 			ohshite(_("unable to lock %s"), desc);
@@ -200,12 +256,15 @@ file_lock(int *lockfd, enum file_lock_flags flags, const char *filename,
 		            "See <https://wiki.debian.org/Teams/Dpkg/FAQ#db-lock>.");
 
 		file_lock_setup(&fl, F_WRLCK);
-		if (fcntl(*lockfd, F_GETLK, &fl) == -1)
+		if (fcntl(*lockfd, F_GETLK, &fl) < 0)
 			ohshit(_("%s was locked by another process\n%s"),
 			       desc, warnmsg);
 
-		ohshit(_("%s was locked by another process with pid %d\n%s"),
-		       desc, fl.l_pid, warnmsg);
+		execname = dpkg_get_pid_execname(fl.l_pid);
+
+		ohshit(_("%s was locked by %s process with pid %d\n%s"),
+		       desc, execname ? execname : C_("process", "<unknown>"),
+		       fl.l_pid, warnmsg);
 	}
 
 	push_cleanup(file_unlock_cleanup, ~0, 3, lockfd, filename, desc);
