@@ -1,4 +1,4 @@
-# Copyright © 2006-2009, 2012-2020, 2022 Guillem Jover <guillem@debian.org>
+# Copyright © 2006-2024 Guillem Jover <guillem@debian.org>
 # Copyright © 2007-2010 Raphaël Hertzog <hertzog@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -14,24 +14,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-package Dpkg::Substvars;
-
-use strict;
-use warnings;
-
-our $VERSION = '2.01';
-
-use Dpkg ();
-use Dpkg::Arch qw(get_host_arch);
-use Dpkg::Vendor qw(get_current_vendor);
-use Dpkg::Version;
-use Dpkg::ErrorHandling;
-use Dpkg::Gettext;
-
-use parent qw(Dpkg::Interface::Storable);
-
-my $maxsubsts = 50;
-
 =encoding utf8
 
 =head1 NAME
@@ -44,11 +26,29 @@ It provides a class which is able to substitute variables in strings.
 
 =cut
 
+package Dpkg::Substvars 2.02;
+
+use strict;
+use warnings;
+
+use Dpkg ();
+use Dpkg::Arch qw(get_host_arch);
+use Dpkg::Vendor qw(get_current_vendor);
+use Dpkg::Version;
+use Dpkg::ErrorHandling;
+use Dpkg::Gettext;
+
+use parent qw(Dpkg::Interface::Storable);
+
+my $maxsubsts = 50;
+
 use constant {
     SUBSTVAR_ATTR_USED => 1,
     SUBSTVAR_ATTR_AUTO => 2,
     SUBSTVAR_ATTR_AGED => 4,
     SUBSTVAR_ATTR_OPT  => 8,
+    SUBSTVAR_ATTR_DEEP => 16,
+    SUBSTVAR_ATTR_REQ  => 32,
 };
 
 =head1 METHODS
@@ -104,6 +104,7 @@ sub set {
     my ($self, $key, $value, $attr) = @_;
 
     $attr //= 0;
+    $attr |= SUBSTVAR_ATTR_DEEP if length $value && $value =~ m{\$};
 
     $self->{vars}{$key} = $value;
     $self->{attr}{$key} = $attr;
@@ -190,12 +191,14 @@ sub parse {
 
 	next if m/^\s*\#/ || !m/\S/;
 	s/\s*\n$//;
-	if (! m/^(\w[-:0-9A-Za-z]*)(\?)?\=(.*)$/) {
+	if (! m/^(\w[-:0-9A-Za-z]*)([?!])?\=(.*)$/) {
 	    error(g_('bad line in substvars file %s at line %d'),
 		  $varlistfile, $.);
 	}
+        ## no critic (RegularExpressions::ProhibitCaptureWithoutTest)
         if (defined $2) {
             $attr = (SUBSTVAR_ATTR_USED | SUBSTVAR_ATTR_OPT) if $2 eq '?';
+            $attr = (SUBSTVAR_ATTR_REQ) if $2 eq '!';
         }
         $self->set($1, $3, $attr);
         $count++;
@@ -299,7 +302,7 @@ sub set_desc_substvars {
 
 =item $s->set_field_substvars($ctrl, $prefix)
 
-Defines field variables from a Dpkg::Control object, with each variable
+Defines field variables from a L<Dpkg::Control> object, with each variable
 having the form "${$prefix:$field}".
 
 They will never be warned about when unused.
@@ -322,29 +325,30 @@ Substitutes variables in $string and return the result in $newstring.
 
 sub substvars {
     my ($self, $v, %opts) = @_;
+    my %seen;
     my $lhs;
     my $vn;
     my $rhs = '';
-    my $count = 0;
     $opts{msg_prefix} //= $self->{msg_prefix};
     $opts{no_warn} //= 0;
 
     while ($v =~ m/^(.*?)\$\{([-:0-9a-z]+)\}(.*)$/si) {
-        # If we have consumed more from the leftover data, then
-        # reset the recursive counter.
-        $count = 0 if (length($3) < length($rhs));
-
-        if ($count >= $maxsubsts) {
-            error($opts{msg_prefix} .
-                  g_("too many substitutions - recursive ? - in '%s'"), $v);
-        }
         $lhs = $1;
         $vn = $2;
         $rhs = $3;
+
         if (defined($self->{vars}{$vn})) {
             $v = $lhs . $self->{vars}{$vn} . $rhs;
             $self->mark_as_used($vn);
-            $count++;
+
+            if ($self->{attr}{$vn} & SUBSTVAR_ATTR_DEEP) {
+                $seen{$vn}++;
+            }
+            if (exists $seen{$vn} && $seen{$vn} >= $maxsubsts) {
+                error($opts{msg_prefix} .
+                      g_("too many \${%s} substitutions (recursive?) in '%s'"),
+                      $vn, $v);
+            }
 
             if ($self->{attr}{$vn} & SUBSTVAR_ATTR_AGED) {
                 error($opts{msg_prefix} .
@@ -376,9 +380,16 @@ sub warn_about_unused {
         # that they are not required in the current situation
         # (example: debhelper's misc:Depends in many cases)
         next if $self->{vars}{$vn} eq '';
-        warning($opts{msg_prefix} .
-                g_('substitution variable ${%s} unused, but is defined'),
-                $vn);
+
+        if ($self->{attr}{$vn} & SUBSTVAR_ATTR_REQ) {
+            error($opts{msg_prefix} .
+                  g_('required substitution variable ${%s} not used'),
+                  $vn);
+        } else {
+            warning($opts{msg_prefix} .
+                    g_('substitution variable ${%s} unused, but is defined'),
+                    $vn);
+        }
     }
 }
 
@@ -394,12 +405,27 @@ sub set_msg_prefix {
     $self->{msg_prefix} = $prefix;
 }
 
-=item $s->filter(remove => $rmfunc)
+=item $s->filter(%opts)
 
-=item $s->filter(keep => $keepfun)
+Filter the substitution variables.
 
-Filter the substitution variables, either removing or keeping all those
-that return true when $rmfunc->($key) or $keepfunc->($key) is called.
+Options:
+
+=over
+
+=item B<remove>
+
+A function returning a boolean,
+used to decide whether to remove a substitution variable,
+executed as $opts{remove}->($var).
+
+=item B<keep>
+
+A function returning a boolean,
+used to decide whether to keep the substitution variable,
+executed as $opts{keep}->($var).
+
+=back
 
 =cut
 
@@ -432,7 +458,14 @@ sub output {
     # Store all non-automatic substitutions only
     foreach my $vn (sort keys %{$self->{vars}}) {
 	next if $self->{attr}{$vn} & SUBSTVAR_ATTR_AUTO;
-        my $op = $self->{attr}{$vn} & SUBSTVAR_ATTR_OPT ? '?=' : '=';
+        my $op;
+        if ($self->{attr}{$vn} & SUBSTVAR_ATTR_OPT) {
+            $op = '?=';
+        } elsif ($self->{attr}{$vn} & SUBSTVAR_ATTR_REQ) {
+            $op = '!=';
+        } else {
+            $op = '=';
+        }
         my $line = "$vn$op" . $self->{vars}{$vn} . "\n";
 	print { $fh } $line if defined $fh;
 	$str .= $line;
@@ -449,6 +482,10 @@ indicated file.
 
 =head1 CHANGES
 
+=head2 Version 2.02 (dpkg 1.22.7)
+
+New feature: Add support for required substitution variables.
+
 =head2 Version 2.01 (dpkg 1.21.8)
 
 New feature: Add support for optional substitution variables.
